@@ -2,12 +2,18 @@ use ansi_term::Color;
 use ansi_term::Color::Fixed;
 use ansi_term::Style;
 use regex::Regex;
+use serde::{Serialize, Deserialize};
 use std::cmp::min;
+use std::convert::From;
+use std::env;
 use std::fmt;
+use std::fs;
 use std::fs::File;
 use std::io::Read;
 use std::num::NonZeroUsize;
 use std::path::Path;
+use std::path::PathBuf;
+use thiserror::Error;
 use unicode_segmentation::UnicodeSegmentation;
 
 
@@ -73,6 +79,38 @@ pub const COLOR_ASCII_OTHER: Color = Color::Purple;
 pub const COLOR_NONASCII: Color = Color::Yellow;
 
 
+/// Per [XDG Base Directory Specification], dotfiles should go to
+/// $XDG_CONFIG_HOME if it exists, and $HOME/.config if it doesn't.
+///
+/// [XDG Base Directory Specification]: https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
+pub fn preferences_file_path() -> PathBuf {
+    let xdg_config_home = env::var("XDG_CONFIG_HOME");
+    let home = env::var("HOME");
+
+    let edhex_s = String::from("edhex");
+    let prefs_s = String::from("preferences");
+
+    if xdg_config_home.is_ok() {
+        let mut return_if_good = PathBuf::from(xdg_config_home.unwrap());
+
+        /* Spec says it must be absolute or ignored */
+        if return_if_good.is_absolute() {
+            return_if_good.push(edhex_s);
+            return_if_good.push(prefs_s);
+            return return_if_good;
+        }
+    }
+
+    let dotconf_s = String::from(".config");
+    if home.is_ok() {
+        [home.unwrap(), dotconf_s, edhex_s, prefs_s].iter().collect()
+    }
+    else {
+        [".".to_owned(), dotconf_s, edhex_s, prefs_s].iter().collect()
+    }
+}
+
+
 /// TODO Per https://doc.rust-lang.org/std/io/enum.ErrorKind.html
 /// IsADirectory isn't available yet.  Check against it when it is.
 pub fn is_a_regular_file(filename: &str) -> bool {
@@ -105,14 +143,32 @@ pub fn num_bytes_or_die(open_file: &Option<std::fs::File>) -> Result<usize, i32>
 }
 
 
-pub fn all_bytes_from_filename(filename: &str) -> Result<Vec<u8>, String> {
+#[derive(Error, Debug)]
+pub enum AllBytesFromFilenameError {
+    #[error("Cannot read file")]
+    FileCannotBeRead,
+    #[error("File does not exist")]
+    FileDoesNotExist,
+    #[error("File is not a regular file")]
+    NotARegularFile,
+    #[error("Cannot read all bytes of file")]
+    CantReadAllBytes,
+}
+
+
+pub fn all_bytes_from_filename(filename: &str)
+        -> Result<Vec<u8>, AllBytesFromFilenameError> {
+
+    /* As written right now, `file` always is Some */
     let file = match filehandle(filename) {
         Ok(Some(filehandle)) => {
             Some(filehandle)
         },
-        Ok(None) => None,
-        Err(error) => {
-            return Err(format!("Problem opening '{}' ({:?})", filename, error));
+        Ok(None) => {
+            return Err(AllBytesFromFilenameError::FileDoesNotExist);
+        },
+        Err(_) => {
+            return Err(AllBytesFromFilenameError::FileCannotBeRead);
         }
     };
 
@@ -120,8 +176,8 @@ pub fn all_bytes_from_filename(filename: &str) -> Result<Vec<u8>, String> {
         Ok(num_bytes) => {
             num_bytes
         },
-        Err(errcode) => {
-            return Err(format!("Couldn't read all of {}.  Errorcode {}", filename, errcode));
+        Err(_) => {
+            return Err(AllBytesFromFilenameError::FileCannotBeRead);
         }
     };
 
@@ -133,22 +189,19 @@ pub fn all_bytes_from_filename(filename: &str) -> Result<Vec<u8>, String> {
             Err(_) => {
                 if path_exists(filename) {
                     if !is_a_regular_file(filename) {
-                        Err(format!("{} isn't a regular file.", filename))
+                        Err(AllBytesFromFilenameError::NotARegularFile)
                     }
                     else {
-                        Err(format!("Couldn't read {}", filename))
+                        Err(AllBytesFromFilenameError::FileCannotBeRead)
                     }
                 }
                 else {
-
-                    /* Empty */
-                    Ok(all_bytes)
+                    Err(AllBytesFromFilenameError::FileDoesNotExist)
                 }
             },
             Ok(num_bytes_read) => {
                 if num_bytes_read != original_num_bytes {
-                    Err(format!("Only read {} of {} bytes of {}",
-                            num_bytes_read, original_num_bytes, filename))
+                    Err(AllBytesFromFilenameError::CantReadAllBytes)
                 }
                 else {
                     Ok(all_bytes)
@@ -157,30 +210,67 @@ pub fn all_bytes_from_filename(filename: &str) -> Result<Vec<u8>, String> {
         }
     }
     else {
-        Ok(all_bytes)
+        Err(AllBytesFromFilenameError::FileDoesNotExist)
     }
 }
 
 
-pub struct State {
+
+/// This struct exists exclusively for serializing State's to disk without
+/// including all_bytes.  If more entries end up in State that need to be
+/// saved off, change this struct to reflect them.
+///
+/// Attempts to deserialize from one of these will cause you to notice
+/// missing keys if State changes without you updating this.
+#[derive(Serialize, Deserialize, Debug)]
+pub struct StateSansBytes {
+    pub prefs: Preferences,
+    pub unsaved_changes: bool,
+    pub filename: String,
+    pub readonly: bool,
+    pub last_search: Option<Vec<u8>>,
+    pub index: usize,
+}
+
+
+impl From<&State> for StateSansBytes {
+    fn from(state: &State) -> Self {
+        StateSansBytes {
+            prefs: state.prefs.clone(),
+            unsaved_changes: state.unsaved_changes,
+            filename: state.filename.clone(),
+            readonly: state.readonly,
+            last_search: state.last_search.clone(),
+            index: state.index,
+        }
+    }
+}
+
+
+impl From<&StateSansBytes> for State {
+    fn from(state_sans_bytes: &StateSansBytes) -> Self {
+        State {
+            prefs: state_sans_bytes.prefs.clone(),
+            unsaved_changes: state_sans_bytes.unsaved_changes,
+            filename: state_sans_bytes.filename.clone(),
+            readonly: state_sans_bytes.readonly,
+            index: state_sans_bytes.index,
+            last_search: state_sans_bytes.last_search.clone(),
+            all_bytes: vec![],
+        }
+    }
+}
+
+
+#[derive(Serialize, Deserialize, Debug, Clone)]
+pub struct Preferences {
     pub radix: u32,
     pub show_byte_numbers: bool,
     pub show_chars: bool,
-    pub unsaved_changes: bool,
-    pub filename: String,
-
     pub show_prompt: bool,
     pub color: bool,
-    pub readonly: bool,
-
-    /* Current byte number, 0 to (len - 1) */
-    pub index: usize,
-
     pub width: NonZeroUsize,
 
-    /* The bytes in memory */
-    pub all_bytes: Vec<u8>,
-
     /* Spaces to put between a byte number and a byte when displaying */
     pub n_padding: String,
 
@@ -189,12 +279,59 @@ pub struct State {
 
     /* Number of lines to print after current line */
     pub after_context: usize,
-
-    pub last_search: Option<Vec<u8>>,
 }
 
+
+impl Preferences {
+    pub fn write_to_disk(&self, filename: &str) -> Result<(), String> {
+        let serialized = serde_json::to_string_pretty(&self);
+
+        if serialized.is_err() {
+            return Err("? Could not serialize preferences.".to_owned());
+        }
+        let serialized = serialized.unwrap();
+
+        let result = std::fs::write(filename, &serialized);
+
+        if result.is_err() {
+            Err(format!("? Couldn't write to {}", filename))
+        }
+        else {
+            Ok(())
+        }
+    }
+
+
+    pub fn read_from_disk(filename: &str) -> Result<Preferences, String> {
+        if let Ok(from_disk) = fs::read_to_string(Path::new(filename)) {
+            serde_json::from_str(&from_disk).or_else(
+                |_| Err(format!("Could not read preferences from {}", filename))
+            )
+        }
+        else {
+            Err(format!("Couldn't read {}", filename))
+        }
+    }
+}
+
+
+pub struct State {
+    pub prefs: Preferences,
+    pub unsaved_changes: bool,
+    pub filename: String,
+    pub readonly: bool,
+    pub last_search: Option<Vec<u8>>,
+
+    /* Current byte number, 0 to (len - 1) */
+    pub index: usize,
+
+    /* The bytes in memory */
+    pub all_bytes: Vec<u8>,
+}
+
+
 pub fn lino(state:&State) -> String {
-    hex_unless_dec_with_radix(state.index, state.radix)
+    hex_unless_dec_with_radix(state.index, state.prefs.radix)
 }
 
 
@@ -233,8 +370,8 @@ pub fn hex_unless_dec_with_radix(number:usize, radix:u32) -> String {
 impl fmt::Debug for State {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "radix: {}|unsaved_changes: {}|show_byte_numbers: {}|show_chars: {}|index: {}|width: {}|n_padding: '{}'|filename: {}|",
-                self.radix, self.unsaved_changes, self.show_byte_numbers,
-                self.show_chars, self.index, self.width, self.n_padding,
+                self.prefs.radix, self.unsaved_changes, self.prefs.show_byte_numbers,
+                self.prefs.show_chars, self.index, self.prefs.width, self.prefs.n_padding,
                 self.filename)
     }
 }
@@ -246,18 +383,79 @@ impl State {
         // TODO Do this with a generator once
         // those are supported in Rust
         let mut to_return:Vec<usize> = vec![];
-        for i in (from..self.all_bytes.len()).step_by(usize::from(self.width)) {
+        for i in (from..self.all_bytes.len()).step_by(usize::from(self.prefs.width)) {
             to_return.push(i);
         }
         to_return
     }
 
 
-    /// Return the range `self.width` bytes starting at `address`.
+    /// Note:  For the forseeable future, from_reader is actually slower
+    /// than drawing the entire file into memory.  So until that changes,
+    /// doing the latter:  https://github.com/serde-rs/json/issues/160
+    pub fn from_filename(filename: &str) -> Result<Self, String> {
+        let state_sans_bytes_s = fs::read_to_string(filename);
+        if state_sans_bytes_s.is_err() {
+            return Err(format!("Problem opening '{}' ({:?})", filename,
+                    state_sans_bytes_s));
+        }
+        let state_sans_bytes_s = state_sans_bytes_s.unwrap();
+        let state_sans_bytes_r = serde_json::from_str(&state_sans_bytes_s);
+        if state_sans_bytes_r.is_err() {
+            return Err(format!("? {:?}", state_sans_bytes_r));
+        }
+        let state_sans_bytes: StateSansBytes = state_sans_bytes_r.unwrap();
+
+        let all_bytes = all_bytes_from_filename(&state_sans_bytes.filename); 
+        if all_bytes.is_ok() {
+            let mut to_return = State::from(&state_sans_bytes);
+            to_return.all_bytes = all_bytes.unwrap();
+            Ok(to_return)
+        }
+        else {
+            Err(format!(
+                "State in file '{}' said to find the bytes in '{}', but
+                    couldn't read '{}'.", filename, state_sans_bytes.filename,
+                    state_sans_bytes.filename))
+        }
+    }
+
+
+    /// Serialize to a json blob.
+    /// Don't include all_bytes, not only because it could be huge, but
+    /// because if you load a state from disk and its bytes differ from
+    /// the actual file's on disk bytes, neither collection of bytes are
+    /// canonical.  User's can write out bytes to other files at will.
+    pub fn write_to_disk(&self, filename: &str) -> Result<(), String> {
+
+        /* NOTICE: To exclude all_bytes, we have to list all the keys
+        *  we want to save.  There is no reasonable, general way to
+        *  iterate through all keys in a struct.
+        *  When State changes what you'd want to save off, you have to
+        *  change what this function saves off, too. */
+        let serialized = serde_json::to_string_pretty(&StateSansBytes::from(self));
+
+        if serialized.is_err() {
+            return Err("? Could not serialize state.".to_owned());
+        }
+        let serialized = serialized.unwrap();
+
+        let result = std::fs::write(filename, &serialized);
+
+        if result.is_err() {
+            Err(format!("? Couldn't write to {}", filename))
+        }
+        else {
+            Ok(())
+        }
+    }
+
+
+    /// Return the range `self.prefs.width` bytes starting at `address`.
     /// Could be cut short by hitting end of all bytes.
     /// Could be empty because `address` is past end of all bytes.
     pub fn bytes_range_from(&self, address:usize) -> std::ops::Range<usize> {
-        let width = usize::from(self.width);
+        let width = usize::from(self.prefs.width);
         if address < self.all_bytes.len() {
             let end_index = min(self.all_bytes.len(), address + width);
             address..end_index
@@ -268,7 +466,7 @@ impl State {
     }
 
 
-    /// Print `self.width` bytes from `address` or cut off if at end of all bytes
+    /// Print `self.prefs.width` bytes from `address` or cut off if at end of all bytes
     pub fn bytes_from(&self, address:usize) -> &[u8] {
         &self.all_bytes[self.bytes_range_from(address)]
     }
@@ -292,7 +490,7 @@ impl State {
         else {
             " "
         };
-        bytes_line_bytes(bytes, line_number, self.width).iter().map(|x| formatted_byte(*x, self.color, underline)).collect::<Vec<String>>().join(join_char)
+        bytes_line_bytes(bytes, line_number, self.prefs.width).iter().map(|x| formatted_byte(*x, self.prefs.color, underline)).collect::<Vec<String>>().join(join_char)
     }
 
 
@@ -300,7 +498,7 @@ impl State {
     // have failed.
     pub fn bytes_line_padding(&self, bytes:&[u8], line_num:usize) -> String{
         let mut to_return = String::new();
-        let expected_length = usize::from(self.width) * 3 - 1;
+        let expected_length = usize::from(self.prefs.width) * 3 - 1;
         for _ in num_graphemes(&self.bytes_line(bytes, line_num, false))..expected_length {
             to_return += " ";
         }
@@ -329,7 +527,7 @@ impl State {
         }
         let max = max.unwrap();
 
-        Ok(min(max, self.index + usize::from(self.width) - 1))
+        Ok(min(max, self.index + usize::from(self.prefs.width) - 1))
     }
 
 
@@ -351,7 +549,7 @@ impl State {
         // [a..b] before_context bytes
         // [c..d]  actual bytes
         // [e..f]  after_context bytes
-        let a = self.index.saturating_sub(self.before_context * usize::from(self.width));
+        let a = self.index.saturating_sub(self.prefs.before_context * usize::from(self.prefs.width));
         let b = self.index.saturating_sub(1);
         let cd = self.range_from_current_row();
         if cd.is_err() {
@@ -359,19 +557,19 @@ impl State {
         }
         let (c, d) = cd.unwrap();
         let e = d + 1;
-        let f = min(max, self.index + (self.after_context + 1) * usize::from(self.width) - 1);
+        let f = min(max, self.index + (self.prefs.after_context + 1) * usize::from(self.prefs.width) - 1);
 
         /* Call the more specific function */
-        if self.before_context > 0 && self.index > 0 {
+        if self.prefs.before_context > 0 && self.index > 0 {
             self.print_bytes_sans_context((a, b), false);
         }
         self.print_bytes_sans_context((c, d),
-                self.before_context > 0 || self.after_context > 0);
-        if self.after_context > 0 {
+                self.prefs.before_context > 0 || self.prefs.after_context > 0);
+        if self.prefs.after_context > 0 {
             self.print_bytes_sans_context((e, f), false);
         }
 
-        return Some(min(max, self.index + usize::from(self.width)));
+        return Some(min(max, self.index + usize::from(self.prefs.width)));
     }
 
 
@@ -406,16 +604,16 @@ impl State {
         }
         let bytes = bytes.unwrap();
 
-        let max_bytes_line_num = max_bytes_line_num(bytes, self.width);
+        let max_bytes_line_num = max_bytes_line_num(bytes, self.prefs.width);
         let addresses = self.addresses(range.0);
         if addresses.len() == 0 {
             return None;
         }
 
         for bytes_line_num in 0..=max_bytes_line_num {
-            if self.show_byte_numbers {
+            if self.prefs.show_byte_numbers {
                 let to_display = address_display(addresses[bytes_line_num],
-                        self.radix, &self.n_padding, underline);
+                        self.prefs.radix, &self.prefs.n_padding, underline);
                 print!("{}|", to_display);
             }
 
@@ -425,9 +623,9 @@ impl State {
                 self.bytes_line_padding(bytes, bytes_line_num)
             );
 
-            if self.show_chars {
-                print!("|   {}", chars_line(bytes, bytes_line_num, self.width,
-                    self.color, underline));
+            if self.prefs.show_chars {
+                print!("|   {}", chars_line(bytes, bytes_line_num, self.prefs.width,
+                    self.prefs.color, underline));
             }
 
             println!();
@@ -442,7 +640,7 @@ impl State {
     }
 
     pub fn range(&self) -> (usize, usize) {
-        (self.index, self.index + usize::from(self.width) - 1)
+        (self.index, self.index + usize::from(self.prefs.width) - 1)
     }
 
     pub fn max_index(&self) -> Result<usize, String> {
@@ -461,27 +659,27 @@ impl fmt::Display for State {
         let mut to_write:String;
         to_write = format!("Filename: {}\n", self.filename);
         to_write += &format!("At byte {} of {}\n", lino(&self),
-                hex_unless_dec_with_radix(self.all_bytes.len(), self.radix));
-        if self.show_byte_numbers {
+                hex_unless_dec_with_radix(self.all_bytes.len(), self.prefs.radix));
+        if self.prefs.show_byte_numbers {
             to_write += &format!("Printing byte numbers in {}\n",
-                string_from_radix(self.radix));
+                string_from_radix(self.prefs.radix));
         };
-        if self.show_chars {
+        if self.prefs.show_chars {
             to_write += &format!("Printing char representations after bytes\n");
         };
         to_write += &format!("Interpreting input numbers as {}\n",
-                string_from_radix(self.radix));
+                string_from_radix(self.prefs.radix));
         to_write += &format!("Printing a newline every {} bytes\n",
-                hex_unless_dec_with_radix(usize::from(self.width), self.radix));
+                hex_unless_dec_with_radix(usize::from(self.prefs.width), self.prefs.radix));
 
-        if self.before_context > 0 {
+        if self.prefs.before_context > 0 {
             to_write += &format!("Printing {} lines before current line\n",
-            hex_unless_dec_with_radix(self.before_context, self.radix));
+            hex_unless_dec_with_radix(self.prefs.before_context, self.prefs.radix));
         };
 
-        if self.after_context > 0 {
+        if self.prefs.after_context > 0 {
             to_write += &format!("Printing {} lines after current line\n",
-            hex_unless_dec_with_radix(self.after_context, self.radix));
+            hex_unless_dec_with_radix(self.prefs.after_context, self.prefs.radix));
         };
 
         if self.unsaved_changes {
@@ -595,7 +793,7 @@ pub fn move_to(state:&mut State, index:usize) -> Result<usize, String> {
         };
 
         if index > _max_index {
-            Err(format!("{} > {} = maximum index", hex_unless_dec_with_radix(index, state.radix), hex_unless_dec_with_radix(_max_index, state.radix)))
+            Err(format!("{} > {} = maximum index", hex_unless_dec_with_radix(index, state.prefs.radix), hex_unless_dec_with_radix(_max_index, state.prefs.radix)))
         }
         else {
             state.index = index;
@@ -634,22 +832,23 @@ mod tests {
                 0x12,
             ];
         let mut state = State {
-            radix: 16,
-            show_byte_numbers: true,
-            show_chars: true,
+            prefs: Preferences {
+                radix: 16,
+                show_byte_numbers: true,
+                show_chars: true,
+                show_prompt: true,
+                color: true,
+                width: NonZeroUsize::new(0x10).unwrap(),
+                n_padding: "   ".to_owned(),
+                after_context: 0,
+                before_context: 0,
+            },
             unsaved_changes: true,
+            index: 0,
             readonly: false,
             last_search: None,
             filename: "filename".to_owned(),
-            show_prompt: true,
-            color: true,
-            index: 0,
-            // height: NonZeroUsize::new(1).unwrap(),
-            width: NonZeroUsize::new(0x10).unwrap(),
             all_bytes: Vec::from(hex_twelve),
-            n_padding: "   ".to_owned(),
-            after_context: 0,
-            before_context: 0,
         };
 
         assert_eq!(state.bytes_from(0), &hex_twelve[0x00..=0x0f]);
@@ -664,7 +863,7 @@ mod tests {
         assert_eq!(state.bytes_from(17), &hex_twelve[0x11..=0x12]);
         assert_eq!(state.bytes_from(18), &hex_twelve[0x12..=0x12]);
         assert_eq!(state.bytes_from(19), &hex_twelve[0..0]);
-        state.width = NonZeroUsize::new(3).unwrap();
+        state.prefs.width = NonZeroUsize::new(3).unwrap();
         assert_eq!(state.bytes_from(0), &hex_twelve[0x00..=0x02]);
         assert_eq!(state.bytes_from(1), &hex_twelve[0x01..=0x03]);
         assert_eq!(state.bytes_from(11), &hex_twelve[0x0b..=0x0d]);
@@ -676,19 +875,22 @@ mod tests {
     #[test]
     fn test_byte_numbers() {
         let mut state = State {
-            radix: 16,
-            show_byte_numbers: true,
-            show_chars: true,
+            prefs: Preferences {
+                radix: 16,
+                show_byte_numbers: true,
+                show_chars: true,
+                show_prompt: true,
+                color: true,
+                after_context: 0,
+                before_context: 0,
+                width: NonZeroUsize::new(0x10).unwrap(),
+                n_padding: "   ".to_owned(),
+            },
             unsaved_changes: true,
             readonly: false,
             last_search: None,
             filename: "filename".to_owned(),
-            show_prompt: true,
-            color: true,
             index: 0,
-            after_context: 0,
-            before_context: 0,
-            width: NonZeroUsize::new(0x10).unwrap(),
             all_bytes: vec![
                 0x00, 0x01, 0x02,
                 0x03, 0x04, 0x05,
@@ -698,7 +900,6 @@ mod tests {
                 0x0f, 0x10, 0x11,
                 0x12,
             ],
-            n_padding: "   ".to_owned(),
         };
         assert_eq!(state.addresses(0x00),
             vec![0x00, 0x10]);
@@ -706,7 +907,7 @@ mod tests {
             vec![0x01, 0x11]);
         assert_eq!(state.addresses(0x04),
             vec![0x04]);
-        state.width = NonZeroUsize::new(3).unwrap();
+        state.prefs.width = NonZeroUsize::new(3).unwrap();
         assert_eq!(state.addresses(0x04),
             vec![0x04, 0x07, 0x0a, 0x0d, 0x10,]);
     }
@@ -803,12 +1004,13 @@ mod tests {
         let _6 = NonZeroUsize::new(6).unwrap();
         let _7 = NonZeroUsize::new(7).unwrap();
         let _8 = NonZeroUsize::new(8).unwrap();
-        assert_eq!(bytes_line_bytes(&bytes, 0, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 0, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 1, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 1, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _2).to_owned(), vec![]);
+        let _empty: Vec<u8> = vec![];
+        assert_eq!(bytes_line_bytes(&bytes, 0, _1).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 0, _2).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _1).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _2).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _1).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _2).to_owned(), _empty);
         let bytes = vec![8, 6, 7, 5, 3, 0, 9,];
         assert_eq!(bytes_line_bytes(&bytes, 0, _1).to_owned(), vec![8,]);
         assert_eq!(bytes_line_bytes(&bytes, 1, _1).to_owned(), vec![6,]);
@@ -817,41 +1019,41 @@ mod tests {
         assert_eq!(bytes_line_bytes(&bytes, 4, _1).to_owned(), vec![3,]);
         assert_eq!(bytes_line_bytes(&bytes, 5, _1).to_owned(), vec![0,]);
         assert_eq!(bytes_line_bytes(&bytes, 6, _1).to_owned(), vec![9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 7, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 8, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 9, _1).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 7, _1).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 8, _1).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 9, _1).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _2).to_owned(), vec![8, 6,]);
         assert_eq!(bytes_line_bytes(&bytes, 1, _2).to_owned(), vec![7, 5,]);
         assert_eq!(bytes_line_bytes(&bytes, 2, _2).to_owned(), vec![3, 0,]);
         assert_eq!(bytes_line_bytes(&bytes, 3, _2).to_owned(), vec![9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 4, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 5, _2).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _2).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 5, _2).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _3).to_owned(), vec![8, 6, 7,]);
         assert_eq!(bytes_line_bytes(&bytes, 1, _3).to_owned(), vec![5, 3, 0,]);
         assert_eq!(bytes_line_bytes(&bytes, 2, _3).to_owned(), vec![9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 3, _3).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 4, _3).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _3).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _3).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _4).to_owned(), vec![8, 6, 7, 5,]);
         assert_eq!(bytes_line_bytes(&bytes, 1, _4).to_owned(), vec![3, 0, 9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 3, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 4, _4).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _4).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _4).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 4, _4).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _5).to_owned(), vec![8, 6, 7, 5, 3,]);
         assert_eq!(bytes_line_bytes(&bytes, 1, _5).to_owned(), vec![0, 9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _5).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 3, _5).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _5).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _5).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _6).to_owned(), vec![8, 6, 7, 5, 3, 0,]);
         assert_eq!(bytes_line_bytes(&bytes, 1, _6).to_owned(), vec![9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _6).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 3, _6).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _6).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _6).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _7).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 1, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 3, _7).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _7).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _7).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _7).to_owned(), _empty);
         assert_eq!(bytes_line_bytes(&bytes, 0, _8).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
-        assert_eq!(bytes_line_bytes(&bytes, 1, _8).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 2, _8).to_owned(), vec![]);
-        assert_eq!(bytes_line_bytes(&bytes, 3, _8).to_owned(), vec![]);
+        assert_eq!(bytes_line_bytes(&bytes, 1, _8).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 2, _8).to_owned(), _empty);
+        assert_eq!(bytes_line_bytes(&bytes, 3, _8).to_owned(), _empty);
     }
 }
 
